================
Progressive Cars
================

ProCars is a program used to reconstruct Ancestral gene orders as CARs
(Contiguous Ancestral Regions) with a progressive homology-based method.
The method runs from a phylogeny tree (without branch lengths needed) with a marked ancestor
and a block file.


ProCars program
===============

Input
-----

The input files are :

  - a phylogeny tree, with Newick format, and with a "@" at the required ancestor node
  - a block file, containing all blocks position and orientation for all extant genomes of the
    tree

You can find examples of these files in the "Examples" directory, and a description of those
file formats in the ProCARs documentation (see below).

You also have to specify a directory where you want to store all results from ProCars run. If the
specified directory does not already exist, it will be created. If you run ProCars twice on the same result
directory, it will erase the last results and replace them by the new ones.


Output
------

* ``ancestor genome``

The output is a Q-tree (see description in documentation), corresponding 
to the ancestral genome, composed of CARs.
Each CAR is an ordered list of signed blocks composing the CAR.

* ``method steps``

In your result directory, you will find a subfolder called *procars_steps*, where you can find all
intermediate ancestor Q-trees (files S_PQtree_ancestor...), as well as the lists of adjacencies
composing the ancestral genome (files S_Adjacencies_ancestor...).

You can also find a full description of those output files in ProCARs documentation (see below).

Examples
--------

In the *Examples* directory, you will find two examples:

  - Fictive data with few blocks and genomes, in order to test and understand ProCARs method.
  - Real data corresponding to the example given in the paper for the reconstruction of
    the Boreoeutherian ancestor.

You will also find a README.md with more information on these data sets, and explaining how to
run them.

Documentation
-------------

In the *doc* directory, you will find the ProCARs documentation, giving you more information about
ProCARs, such as:

  - input and output file formats
  - all information about installing/uninstalling/running ProCARs
  - information about the example datasets provided with ProCARs
  - ProCARs API

Open *doc/index.html* to see the ProCARs documentation's table of contents.

Using ProCars
=============

To run ProCars, you have two possibilities, depending on your use.

For a classic use of ProCars, you can install the package (in final or deployment mode, see
here-below for more information before installing the package). Then,
from anywhere in your computer, run::

    procars_main -t tree_file -b block_file -r result_directory

If you want to work on the ProCARs code, and do not want to install the package,
you must run ProCARs from its root directory (same directory as *procars*, *bin*, *setup.py* etc.),
and use the following command::

    PYTHONPATH+=. python bin/procars_main -t tree_file -b block_file -r result_directory


ProCars installation (final mode)
=================================

To install the package *procars*, and all its dependances, from the root directory, just do::

    sudo pip install .

or::

    sudo python setup.py install

You will then be able to use ProCARs from any directory in your computer,
just as any other software.

Warning: This must be done only if you downloaded a stable version of the package, and won't
do any more changes on the scripts and modules. Indeed, by installing the package, the 
changes done after won't be taken into account while running the scripts.
If you plan to work on the scripts, or to download a new version after,
choose the deployment installation (see below).


ProCars installation (deployment mode)
======================================

If you want to install the package ProCARs while still working on
modifying the scripts, or being able to download and run latest versions, just do::

    sudo pip install -e .

or::

    sudo python setup.py develop

Your changes will then be taken into account. As you installed the package, you will be able to run
ProCARs from any directory in your computer.


Uninstalling ProCars
====================

Whatever the way you installed ProCARs, you can uninstall it by running::

    sudo pip uninstall procars


Running Tests
=============

If you want to work on ProCars scripts, you can use the tests provided with the software,
used to check each of its functionalities. To run the tests, run, from the root of the project::

    PYTHONPATH+=. py.test test/
