#!/usr/bin/python
# -*- coding: utf-8 -*-:
"""Setup script"""

import procars
try:
    from setuptools import setup
    from setuptools.command.test import test as TestCommand
except ImportError:
    from distutils.core import setup
    from distutils.core import Command as TestCommand


class PyTest(TestCommand):
    def initialize_options(self):
        pass

    def finalize_options(self):
        self.test_args = []
        self.test_suite = True

    def run_tests(self):
        import pytest
        import sys
        errno = pytest.main(self.test_args)
        sys.exit(errno)


def parse_requirements(requirements):
    """
    Returns the list of requirements found in the requirements file
    """
    with open(requirements, "r") as req_file:
        return [l.strip('\n') for l in req_file if l.strip('\n')
                and not l.startswith('#')]

packages = ['procars', 'procars.step_modules', 'procars.utils']
requires = parse_requirements("requirements.txt")
scripts = ['bin/procars_main']

classifiers = [
    "Environment :: Console",
    "Intended Audience :: Science/Research",
    "License :: CeCILL-B",
    "Programming Language :: Python :: 2.7",
    "Operating System :: OS Independent",
    "Topic :: Scientific/Engineering :: Bio-Informatics",
]

with open('README.md') as f:
    long_description = f.read()

setup(
    name='procars',
    packages=packages,
    version=procars.__version__,
    description='ProCARs: Progressive Reconstruction of Ancestral Gene Orders',
    long_description=long_description,
    author='Aïda Ouangraoua, Amandine Perrin',
    author_email='aida.ouangraoua@inria.fr, amandine.perrin@inria.fr',
    license='CeCILL-B',
    platforms='OS Independent',
    package_data={'': ['LICENSE']},
    download_url='http://bioinfo.lifl.fr/procars',
    url='http://bioinfo.lifl.fr/procars',
    scripts=scripts,
    include_package_data=True,
    install_requires=requires,
    tests_require=['pytest'],
    cmdclass={'test': PyTest},
    classifiers=classifiers
)
