# -*- coding: utf-8 -*-:
"""
Functional tests for procars_steps
"""

from procars import procars_steps
import pytest
import os
import shutil


def test_stepa():
    """
    Test on running step a) of ProCARs
    """
    pqtree_file = os.path.join("test", 'data', "res_PQtree")
    adjacency_file = os.path.join("test", "data", "res_teststepa_adjs")
    # create empty adjacency file for iteration 0
    open(adjacency_file + "_0.txt", "w").close()
    step = 0
    tree_bin = os.path.join("test", 'data', "res_tree_information.bin")
    blocks_bin = os.path.join("test", 'data', "res_block_information.bin")
    car_bin = os.path.join("test", 'data', "res_car_information.bin")
    car_neigh_bin = os.path.join("test", 'data', "res_neigh_car_information.bin")
    last_pqtree = pqtree_file + "_0.txt"
    nb_blocks = 8
    res = procars_steps.run_step_a(pqtree_file, adjacency_file, step, tree_bin, blocks_bin,
                                   car_bin, car_neigh_bin, nb_blocks, last_pqtree)
    step, donextstep, nb_discarded, last_pqtree2 = res
    assert step == 1
    assert donextstep
    assert nb_discarded == 2
    assert last_pqtree2 == pqtree_file + "_1.txt"
    os.remove(last_pqtree2)
    os.remove(adjacency_file + "_0.txt")
    os.remove(adjacency_file + "_1.txt")
    os.remove(adjacency_file + "_1.txt_discarded.txt")


def test_stepb():
    """
    Test on running step b) of ProCARs
    """
    pqtree_file = os.path.join("test", 'data', "res_PQtree")
    adjacency_file = os.path.join("test", "data", "res_teststepb_adjs")
    # copy file for adjacencies already added at last step:
    ref_added = os.path.join("test", "data", "res_testcomputepqtree_findcars_2.txt")
    added = os.path.join("test", "data", "res_teststepb_adjs_2.txt")
    shutil.copyfile(ref_added, added)
    # copy file for adjacencies discarded at last step:
    ref_discard = os.path.join("test", "data", "res_testcomputepqtree_findcars_2.txt_discarded.txt")
    discard2 = os.path.join("test", "data", "res_teststepb_adjs_2.txt_discarded.txt")
    discard3 = os.path.join("test", "data", "res_teststepb_adjs_3.txt_discarded.txt")
    shutil.copyfile(ref_discard, discard2)
    shutil.copyfile(ref_discard, discard3)
    step = 3
    tree_bin = os.path.join("test", 'data', "res_tree_information.bin")
    nb_blocks = 9
    res = procars_steps.run_step_b(pqtree_file, adjacency_file, step, tree_bin, nb_blocks)
    donextstep, last_pqtree = res
    assert donextstep
    assert last_pqtree == pqtree_file + "_3.txt"
    with open(last_pqtree, "r") as pqtree:
        reslines = pqtree.readlines()
    assert reslines == ['#CAR1\n', '_Q 1 -3 4 7 -9 -2 5 -8 Q_\n', '#CAR2\n', '_Q 6 Q_\n']
    os.remove(last_pqtree)
    os.remove(discard2)
    os.remove(discard3)
    os.remove(added)
    os.remove(adjacency_file + "_3.txt")
    os.remove(adjacency_file + "_3.txt_discarded_used.txt")


def test_stepc():
    """
    Test on running step c) of ProCARs
    """
    tree_bin = os.path.join('test', 'data', "res_tree_boreo_information.bin")
    blocks_bin = os.path.join('test', 'data', 'res_block_boreo_information.bin')
    neigh_bin = os.path.join('test', 'data', 'res_neigh_dcj.bin')
    car_bin = os.path.join('test', 'data', 'res_cars_dcj.bin')
    adj_file = os.path.join('test', 'data', 'res_teststepc_adj')
    # create empty adjacency file for last adjacencies:
    open(adj_file + "_3.txt", "w").close()
    step = 4
    nb_blocks = 689
    pqtree_file = os.path.join('test', 'data', 'res_teststepc_pqtree')
    last_pqtree = pqtree_file + '_3.txt'
    donextstep = False
    res = procars_steps.run_step_c(pqtree_file, adj_file, step, tree_bin, blocks_bin,
                                   car_bin, neigh_bin, nb_blocks, last_pqtree, donextstep)
    donextstep, last_pqtree2 = res
    assert donextstep
    assert last_pqtree2 == pqtree_file + "_4.txt"
    with open(adj_file + "_4.txt", "r") as resadj:
        adjlines = resadj.readlines()
    assert adjlines == ['50 -545 2 -20 2 4 0 0 0 0 0 0 0 0 0 1 0 1\n',
                        '601 602 23 24 2 4 0 0 0 1 1 1 1 0 0 0 0 0\n',
                        '659 657 29 28 2 4 0 0 0 0 0 0 0 0 0 1 1 0\n']
    os.remove(adj_file + "_3.txt")
    os.remove(adj_file + "_4.txt")
    os.remove(last_pqtree2)
