# -*- coding: utf-8 -*-:
"""
Unit tests for compute_initialpqtree.py
"""
from procars.utils import initial_files_reader
import pytest
import os


# ########################################################################################
# All fixtures needed for tests
@pytest.fixture(scope="module")
def treefile():
    return os.path.join("test", "data", "data_tree.ph")


@pytest.fixture(scope="module")
def expect_tree():
    return {1: ['N1', -1, 2, 7], 2: ['N2', 1, 3, 4], 3: ['C_c', 2, -1, -1], 4: ['N4', 2, 5, 6],
            5: ['A_a', 4, -1, -1], 6: ['B', 4, -1, -1], 7: ['N7', 1, 8, 9],
            8: ['D_species', 7, -1, -1], 9: ['E', 7, -1, -1]}


@pytest.fixture(scope="module")
def blocksfile():
    return os.path.join("test", "data", "data_orthology_blocks.txt")


@pytest.fixture(scope="module")
def expect_blocks():
    return [[[], [[1, 99, 1, 1], [101, 199, 2, 1], [201, 299, 3, 1], [301, 399, 4, 1]],
             [[1, 99, 5, 1], [101, 199, 6, 1], [201, 299, 7, 1], [301, 399, 8, -1]]],
            [[], [[1, 99, 1, 1], [101, 199, 2, 1], [201, 299, 3, 1], [301, 399, 4, 1],
             [401, 499, 5, 1], [501, 599, 6, 1], [601, 699, 7, 1], [701, 799, 8, 1]]],
            [[], [[1, 99, 1, 1], [101, 199, 2, 1], [201, 299, 3, 1], [301, 399, 4, 1],
             [401, 499, 5, 1], [501, 599, 6, 1], [601, 699, 7, 1], [701, 799, 8, -1]]],
            [[], [[1, 99, 1, 1], [101, 199, 2, -1], [201, 299, 3, 1], [301, 399, 5, -1]],
             [[301, 399, 4, 1], [201, 299, 6, -1], [101, 199, 7, -1], [1, 99, 8, 1]]],
            [[], [[1, 99, 1, 1], [101, 199, 2, 1], [201, 299, 3, 1], [301, 399, 5, -1]],
             [[1, 99, 4, -1], [101, 199, 6, 1], [201, 299, 7, 1], [301, 399, 8, 1]]]]


# ########################################################################################
# Unit tests
def test_read_tree_file(treefile, expect_tree):
    """
    Test reading tree file
    """
    tree, ancestor_id = initial_files_reader.read_treefile(treefile)
    assert ancestor_id == 7
    assert tree == expect_tree


def test_read_blocks(blocksfile, expect_blocks):
    """
    Test on reading block file
    """
    nb_species = 5
    spe_ids = {'C_c': 0, 'B': 2, 'E': 4, 'D_species': 3, 'A_a': 1}
    blocks, nb_blocks = initial_files_reader.read_block_file(blocksfile, nb_species, spe_ids)
    assert blocks == expect_blocks
    assert nb_blocks == 8


def test_read_block_line():
    """
    Test on reading one line of a block file
    """
    line = "D_species.1:1-99 +"
    chm = {'X': 0}
    spe_ids = {'C_c': 0, 'B': 2, 'E': 4, 'D_species': 3, 'A_a': 1}
    bloc_id = 3
    blocks = [[] for _ in xrange(5)]
    chms, b_id = initial_files_reader.read_block_line(line, chm, spe_ids, blocks, bloc_id)
    assert b_id == bloc_id
    assert chms == {'1': 1, 'X': 0}
    assert blocks == [[], [], [], [[], [[1, 99, 3, 1]]], []]


def test_read_block_line2(expect_blocks):
    """
    Test on reading one line of a block file
    """
    line = ">5"
    chm = {'X': 0}
    spe_ids = {'C_c': 0, 'B': 2, 'E': 4, 'D_species': 3, 'A_a': 1}
    bloc_id = 3
    blocks = [[] for _ in xrange(5)]
    chms, b_id = initial_files_reader.read_block_line(line, chm, spe_ids, blocks, bloc_id)
    assert b_id == 5
    assert chms == chm
    assert blocks == [[], [], [], [], []]
