# -*- coding: utf-8 -*-:
"""
Unit tests for utils_IO.py
"""

from procars.utils import utils_IO
import pytest
import os
import StringIO
try:
    import cPickle as pickle
except ImportError:
    import pickle


# ########################################################################################
# All fixtures needed for tests
@pytest.fixture(scope="module")
def expect_tree():
    return {1: ['N1', -1, 2, 7], 2: ['N2', 1, 3, 4], 3: ['C_c', 2, -1, -1], 4: ['N4', 2, 5, 6],
            5: ['A_a', 4, -1, -1], 6: ['B', 4, -1, -1], 7: ['N7', 1, 8, 9],
            8: ['D_species', 7, -1, -1], 9: ['E', 7, -1, -1]}


@pytest.fixture(scope="module")
def treebin():
    return os.path.join("test", "data", "res_tree_information.bin")


@pytest.fixture(scope="module")
def blocksbin():
    return os.path.join("test", "data", "res_block_information.bin")


@pytest.fixture(scope="module")
def expect_blocks():
    return [[[], [[1, 99, 1, 1], [101, 199, 2, 1], [201, 299, 3, 1], [301, 399, 4, 1]],
             [[1, 99, 5, 1], [101, 199, 6, 1], [201, 299, 7, 1], [301, 399, 8, -1]]],
            [[], [[1, 99, 1, 1], [101, 199, 2, 1], [201, 299, 3, 1], [301, 399, 4, 1],
             [401, 499, 5, 1], [501, 599, 6, 1], [601, 699, 7, 1], [701, 799, 8, 1]]],
            [[], [[1, 99, 1, 1], [101, 199, 2, 1], [201, 299, 3, 1], [301, 399, 4, 1],
             [401, 499, 5, 1], [501, 599, 6, 1], [601, 699, 7, 1], [701, 799, 8, -1]]],
            [[], [[1, 99, 1, 1], [101, 199, 2, -1], [201, 299, 3, 1], [301, 399, 5, -1]],
             [[1, 99, 8, 1], [101, 199, 7, -1], [201, 299, 6, -1], [301, 399, 4, 1]]],
            [[], [[1, 99, 1, 1], [101, 199, 2, 1], [201, 299, 3, 1], [301, 399, 5, -1]],
             [[1, 99, 4, -1], [101, 199, 6, 1], [201, 299, 7, 1], [301, 399, 8, 1]]]]


@pytest.fixture(scope="module")
def leaves():
    return [3, 5, 6, 8, 9]


# ########################################################################################
# Unit tests on utils_IO.py
def test_save_info():
    """
    Test on saving some information into a binary file
    """
    bin_filename = os.path.join("test", "data", "car_information.bin")
    cars = [[], [1], [2], [3], [4], [5], [6], [7], [8]]
    nb_cars = 8
    block_position_in_car = [0, 0, 0, 0, 0, 0, 0, 0, 0]
    blocks_car = [[[], [1, 2, 3, 4], [5, 6, 7, 8]], [[], [1, 2, 3, 4, 5, 6, 7, 8]],
                  [[], [1, 2, 3, 4, 5, 6, 7, 8]], [[], [1, 2, 3, 5], [8, 7, 6, 4]],
                  [[], [1, 2, 3, 5], [4, 6, 7, 8]]]
    information = [cars, nb_cars, block_position_in_car, blocks_car]
    utils_IO.save_binary_information(bin_filename, information)
    with open(bin_filename, "rb") as carfile:
        my_dep = pickle.Unpickler(carfile)
        info = my_dep.load()
    for num, inf in enumerate(info):
        assert info[num] == information[num]
    os.remove(bin_filename)


def test_save_carsneighbor_info():
    """
    Test on saving all cars neighbors information
    """
    bin_filename = os.path.join("test", "data", "car_neighbor_information.bin")
    left = [[[], [], [1], [2], [3], [], [5], [6], []],
            [[], [], [1], [2], [3], [4], [5], [6], [7]],
            [[], [], [1], [2], [3], [4], [5], [6], []],
            [[], [], [-3], [-2], [-6], [], [-4], [6], []],
            [[], [], [1], [2], [-6], [], [-4], [6], [7]]]
    right = [[[], [2], [3], [4], [], [6], [7], [-8], [-7]],
             [[], [2], [3], [4], [5], [6], [7], [8], []],
             [[], [2], [3], [4], [5], [6], [7], [-8], [-7]],
             [[], [-2], [-1], [-5], [], [-3], [7], [-8], [-7]],
             [[], [2], [3], [-5], [], [-3], [7], [8], []]]
    information = [left, right]
    utils_IO.save_binary_information(bin_filename, information)
    with open(bin_filename, "rb") as carfile:
        my_dep = pickle.Unpickler(carfile)
        info = my_dep.load()
    for num, inf in enumerate(info):
        assert info[num] == information[num]
    os.remove(bin_filename)


def test_save_tree_info(expect_tree):
    """
    Test on saving all tree information into a binary file
    """
    binary_tree_file = os.path.join("test", "data", "tree_information.bin")
    ances_id = 7
    path = [0, 1, 0, 0, 0, 0, 0, 1, 0, 0]
    leaves = [3, 5, 6, 8, 9]
    internal = [4, 2, 1, 7]
    nb_spe = 5
    spe1_id = [0, 1, 2]
    spe2_id = [3]
    spe3_id = [4]
    spe_ids = {'C_c': 0, 'B': 2, 'E': 4, 'D_species': 3, 'A_a': 1}
    information = [expect_tree, ances_id, path, leaves, internal,
                   nb_spe, spe1_id, spe2_id, spe3_id, spe_ids]
    utils_IO.save_binary_information(binary_tree_file, information)
    with open(binary_tree_file, "rb") as treefile:
        my_dep = pickle.Unpickler(treefile)
        info = my_dep.load()
    for num, inf in enumerate(info):
        assert info[num] == information[num]
    os.remove(binary_tree_file)


def test_read_info(expect_tree):
    """
    Test on reading information from a binary file
    """
    bin_filename = os.path.join("test", "data", "res_tree_information.bin")
    info = utils_IO.read_binary_file(bin_filename)
    ances_id = 7
    path = [0, 1, 0, 0, 0, 0, 0, 1, 0, 0]
    leaves = [8, 9, 3, 5, 6]
    internal = [4, 2, 1, 7]
    nb_spe = 5
    spe1_id = [0, 1, 2]
    spe2_id = [3]
    spe3_id = [4]
    spe_ids = {'C_c': 0, 'B': 2, 'E': 4, 'D_species': 3, 'A_a': 1}
    information = [expect_tree, ances_id, path, leaves, internal,
                   nb_spe, spe1_id, spe2_id, spe3_id, spe_ids]
    for num, inf in enumerate(info):
        assert inf == information[num]


def test_read_adj_file():
    """
    Test on reading an adjacency file, and returning all blocks with their left and right neighbors
    """
    adj_filename = os.path.join("test", "data", "res_S_Adjacencies_file.txt")
    nb_blocks = 30
    left, right = utils_IO.read_adjacency_file(adj_filename, nb_blocks)
    assert left == {1: 0, 2: 1, 3: 0, 4: 3, 5: 4, 6: 0, 7: 6, 8: 0, 9: 0, 10: 9, 11: 10,
                    12: 11, 13: 0, 14: -15, 15: -14, 16: 17, 17: -25, 18: 16, 19: 0, 20: 0,
                    21: 0, 22: 0, 23: 14, 24: 0, 25: -17, 26: 0, 27: 0, 28: 0, 29: 0, 30: 0}
    assert right == {1: 2, 2: 0, 3: 4, 4: 5, 5: 0, 6: 7, 7: 0, 8: 0, 9: 10, 10: 11, 11: 12,
                     12: 0, 13: 0, 14: 23, 15: 0, 16: 18, 17: 16, 18: 0, 19: -20, 20: -19,
                     21: -22, 22: -21, 23: 0, 24: 0, 25: 0, 26: 0, 27: 0, 28: -30, 29: 0, 30: -28}


def test_read_discarded():
    """
    Test on reading a discarded adjacency file, and returning blocks with their possible left and
    right neighbors
    """
    adj_filename = os.path.join("test", "data", "res_S_Adjacencies_discarded_file.txt")
    nb_blocks = 6
    left, right = utils_IO.read_adjacency_file(adj_filename, nb_blocks, discarded=True)
    assert left == {1: [2], 2: [], 3: [1], 4: [-5], 5: [4, -6, -4], 6: [-5]}
    assert right == {1: [-2, 3], 2: [-1, 1], 3: [], 4: [5], 5: [], 6: []}


def test_read_car_file():
    """
    Test on reading a txt car file, and parsing it to get all information
    """
    nb_blocks = 8
    filename = os.path.join("test", "data", "res_PQtree_0.txt")
    res = utils_IO.read_car_file(filename, nb_blocks)
    cars, block_2_car, block_position_in_car = res
    assert cars == [[], [1], [2], [3], [4], [5], [6], [7], [8]]
    assert block_2_car == [0, 1, 2, 3, 4, 5, 6, 7, 8]
    assert block_position_in_car == [0] * 9


def test_read_conflict_file(leaves, expect_tree):
    """
    Test on reading the adjacencies discarded at previous step from a txt file
    """
    filename = "res_S_Adjacencies_boreoeutherian_ancestor_2.txt_discarded.txt"
    adj_file = os.path.join("test", "data", filename)
    nb_species = 5
    spe_ids = {'C_c': 0, 'B': 2, 'E': 4, 'D_species': 3, 'A_a': 1}
    iterator = utils_IO.read_conflict_adj_file(adj_file, nb_species, leaves, expect_tree, spe_ids)
    all_labs = [{8: [2], 9: [2], 3: [1], 5: [1], 6: [1]}, {8: [1], 9: [1], 3: [1], 5: [2], 6: [2]},
                {8: [1], 9: [1], 3: [2], 5: [1], 6: [1]}, {8: [2], 9: [2], 3: [1], 5: [1], 6: [1]},
                {8: [1], 9: [1], 3: [1], 5: [1], 6: [1]}, {8: [1], 9: [1], 3: [1], 5: [1], 6: [1]}]
    all_adjs = [(134, -142), (134, 396), (142, 134), (611, 612), (-612, 589), (-134, 396)]
    all_info = [['9', '-10', '1', '2', '0', '0', '0', '1', '1'],
                ['9', '27', '1', '2', '0', '1', '1', '0', '0'],
                ['10', '9', '1', '2', '1', '0', '0', '0', '0'],
                ['31', '-34', '1', '2', '0', '0', '0', '1', '1'],
                ['34', '31', '1', '2', '0', '0', '0', '0', '0'],
                ['-9', '27', '1', '2', '0', '0', '0', '0', '0']]
    cur_id = 0
    for lab, ids, adj, info in iterator:
        assert lab == all_labs[cur_id]
        assert ids == cur_id
        assert adj == all_adjs[cur_id]
        assert info == all_info[cur_id]
        cur_id += 1


def test_write_retained():
    """
    Test on writing retained adjacencies in a txt file
    """
    filename = os.path.join("test", "data", "res_retained_adjs")
    adj_infos = {0: (9, -10, 1, 2, 0, 0, 0, 1, 1),
                 1: (9, 27, 1, 2, 0, 0, 1, 1, 0),
                 2: (10, 9, 1, 2, 0, 0, 1, 1, 0),
                 3: (31, -34, 1, 2, 0, 0, 0, 1, 1),
                 4: (34, 31, 1, 2, 0, 0, 0, 1, 1),
                 5: (-9, 27, 1, 2, 0, 0, 0, 0, 0)}
    maximum_set = [0, 3, 5]
    adj_ids = {0: (134, -142), 1: (134, 396), 2: (142, 134), 3: (611, 612),
               4: (-612, 589), 5: (-134, 396)}
    utils_IO.write_retained_conflict_adjs(filename, adj_infos, maximum_set, adj_ids)
    reffile = os.path.join("test", "data", "res_S_Adjacencies_boreoeutherian_ancestor_2.txt")
    with open(filename, "r") as resfile:
        result = resfile.readlines()
    with open(reffile, "r") as ref:
        reference = ref.readlines()
    for num, line in enumerate(result):
        assert line.strip() == reference[num].strip()
    os.remove(filename)


def test_write_adjacency():
    """
    Test on writing a given adjacency information into a file
    """
    output_file = StringIO.StringIO("")
    cars = [[], [1], [2, 3], [4], [5], [6], [7], [8]]
    car_adjacency = (2, 3)
    adj_type = 1
    step_nb = 2
    labels = ""
    utils_IO.write_adjacency(output_file, cars, car_adjacency, adj_type, step_nb, labels)
    assert output_file.getvalue() == "3 4 2 3 1 2\n"


def test_write_car_file():
    """
    Test on writing all cars into a file
    """
    car_filename = os.path.join("test", "data", "utils_IO.testwritecarfile.txt")
    all_cars = [[1, 2], [3, -4, -5, 7], [8, 10, -9], [-6]]
    utils_IO.write_car_file(car_filename, all_cars)
    with open(car_filename, 'r') as carfile:
        alllines = carfile.readlines()
    expect_lines = ["#CAR1\n", "_Q 1 2 Q_\n", "#CAR2\n", "_Q 3 -4 -5 7 Q_\n", "#CAR3\n",
                    "_Q 8 10 -9 Q_\n", "#CAR4\n", "_Q -6 Q_\n"]
    assert alllines == expect_lines
    os.remove(car_filename)
